//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
var script$1 = {
  name: "Draggable",
  props: {
    classOnDrag: String,
    payload: {
      type: Object,
      default: {}
    }
  },
  emits: ["startDrag", "endDrag"],

  data() {
    return {
      dragInProgress: false,
      originalPosition: {},
      originalSize: {
        height: "100%",
        width: "100%",
        sizeInPc: true
      },
      mouseCoordinates: {
        x: 0,
        y: 0
      },
      eventListeners: []
    };
  },

  computed: {
    elementWidth() {
      return this.$refs.draggable.offsetWidth;
    },

    elementHeight() {
      return this.$refs.draggable.offsetHeight;
    },

    left() {
      return this.$refs.draggable.offsetLeft;
    },

    right() {
      return this.$refs.draggable.offsetLeft + this.$refs.draggable.offsetWidth;
    },

    top() {
      return this.$refs.draggable.offsetTop;
    },

    bottom() {
      return this.$refs.draggable.offsetTop + this.$refs.draggable.offsetHeight;
    }

  },
  methods: {
    emit(eventName) {
      this.$emit(eventName, this.payload);
    },

    emitDragStart() {
      const startEvent = new CustomEvent("CustomDragStart");
      document.dispatchEvent(startEvent);
      this.emit("startDrag");
    },

    emitDragEnd(x, y) {
      const detail = {
        position: {
          x,
          y
        },
        payload: this.payload
      };
      this.emit("endDrag");
      const dropEvent = new CustomEvent("CustomDragEnd", {
        detail
      });
      document.dispatchEvent(dropEvent);
    },

    setOriginalPositionAndDimensions() {
      this.originalPosition.left = this.$refs.draggable.offsetLeft;
      this.originalPosition.top = this.$refs.draggable.offsetTop;
      this.originalSize.height = this.elementHeight;
      this.originalSize.width = this.elementWidth;
      this.originalSize.sizeInPc = false;
    },

    move(_ref) {
      let {
        pageX,
        pageY
      } = _ref;
      if (!this.dragInProgress) return;
      this.setElementPosition(pageY - this.mouseCoordinates.y + "px", pageX - this.mouseCoordinates.x + "px");
    },

    areCoordinatesInElement(x, y) {
      const horizontal = x > this.left && x < this.right;
      const vertical = y > this.top && y < this.bottom;
      return horizontal && vertical;
    },

    setElementPosition(top, left) {
      this.$refs.draggable.style.top = top;
      this.$refs.draggable.style.left = left;
      this.$refs.draggable.style.height = this.originalSize.height + "px";
      this.$refs.draggable.style.width = this.originalSize.width + "px";
    },

    startHandler(_ref2) {
      let {
        pageX,
        pageY
      } = _ref2;
      this.dragInProgress = true;
      this.$refs.draggable.style.height = this.originalSize.height + "px";
      this.$refs.draggable.style.width = this.originalSize.width + "px";
      this.setMouseCoordinates({
        pageX,
        pageY
      });
      this.emitDragStart();
    },

    endHandler(_ref3) {
      let {
        pageX,
        pageY
      } = _ref3;
      this.dragInProgress = false;
      if (this.originalPosition.left !== undefined && this.originalPosition.top !== undefined) this.setElementPosition(this.originalPosition.top + "px", this.originalPosition.left + "px");
      this.mouseCoordinates.x = 0;
      this.mouseCoordinates.y = 0;
      this.emitDragEnd(pageX, pageY);
    },

    touchStart(e) {
      if (!e.targetTouches || e.targetTouches.length !== 1) return;
      this.startHandler(e.targetTouches[0]);
    },

    touchMove(event) {
      if (!event.targetTouches) return;
      if (event.targetTouches.length !== 1) return;
      const touchLocation = event.targetTouches[0];
      this.move(touchLocation);
    },

    touchEnd(event) {
      if (!event.changedTouches || event.changedTouches.length !== 1) return;
      this.endHandler(event.changedTouches[0]);
    },

    setMouseCoordinates(e) {
      this.mouseCoordinates.x = e.pageX - this.left;
      this.mouseCoordinates.y = e.pageY - this.top;
    },

    setup() {
      window.addEventListener("resize", this.setOriginalPositionAndDimensions);
      this.eventListeners.push(document.addEventListener("mousedown", e => {
        if (e.button !== 0) return;
        if (!this.areCoordinatesInElement(e.x, e.y)) return;
        e.preventDefault();
        this.startHandler(e);
      }));
      this.eventListeners.push(document.addEventListener("mouseup", e => {
        if (!this.dragInProgress) return;
        this.endHandler(e);
      }));
      this.eventListeners.push(document.addEventListener("mousemove", this.move));
    }

  },

  mounted() {
    this.setOriginalPositionAndDimensions();
    this.setup();
    this.$refs.draggable.childNodes.forEach(node => {
      node.setAttribute("draggable", "false");
    });
  },

  unmounted() {
    window.removeEventListener("resize");
    this.eventListeners.forEach(l => document.removeEventListener(l));
  }

};

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

/* script */
const __vue_script__$1 = script$1;
/* template */

var __vue_render__$1 = function () {
  var _obj;

  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', _vm._b({
    ref: "draggable",
    class: (_obj = {}, _obj[_vm.classOnDrag] = _vm.dragInProgress, _obj),
    style: {
      display: 'flex',
      height: '100%',
      width: '100%',
      position: _vm.dragInProgress ? 'absolute' : undefined
    },
    attrs: {
      "custom-draggable": "true",
      "drag-in-progress": _vm.dragInProgress
    },
    on: {
      "touchstart": _vm.touchStart,
      "touchend": _vm.touchEnd,
      "touchmove": _vm.touchMove
    }
  }, 'div', _vm.$attrs, false), [_vm._t("default")], 2);
};

var __vue_staticRenderFns__$1 = [];
/* style */

const __vue_inject_styles__$1 = undefined;
/* scoped */

const __vue_scope_id__$1 = undefined;
/* module identifier */

const __vue_module_identifier__$1 = undefined;
/* functional template */

const __vue_is_functional_template__$1 = false;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

const __vue_component__$2 = /*#__PURE__*/normalizeComponent({
  render: __vue_render__$1,
  staticRenderFns: __vue_staticRenderFns__$1
}, __vue_inject_styles__$1, __vue_script__$1, __vue_scope_id__$1, __vue_is_functional_template__$1, __vue_module_identifier__$1, false, undefined, undefined, undefined);

var __vue_component__$3 = __vue_component__$2;

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
var script = {
  name: "Dropable",
  emits: ["onDrop"],
  props: {
    dragOverClass: String
  },

  data() {
    return {
      eventListeners: [],
      isBeingDraggedOver: false,
      isGlobalDragInProgress: false
    };
  },

  computed: {
    left() {
      return this.$refs.dropable.offsetLeft;
    },

    right() {
      return this.$refs.dropable.offsetLeft + this.$refs.dropable.offsetWidth;
    },

    top() {
      return this.$refs.dropable.offsetTop;
    },

    bottom() {
      return this.$refs.dropable.offsetTop + this.$refs.dropable.offsetHeight;
    }

  },
  methods: {
    dragenterHandler() {
      this.isBeingDraggedOver = true;
    },

    dragleaveHandler(event) {
      if (event.pageX && event.pageY && this.areCoordinatesInElement(event.pageX, event.pageY)) return;
      this.isBeingDraggedOver = false;
    },

    dropHandler(event) {
      event.preventDefault();
      this.isBeingDraggedOver = false;
      const pl = event.dataTransfer.getData("payload");
      let payload;
      if (pl) payload = JSON.parse(pl);
      this.$emit("onDrop", payload);
    },

    areCoordinatesInElement(x, y) {
      const horizontal = x > this.left && x < this.right;
      const vertical = y > this.top && y < this.bottom;
      return horizontal && vertical;
    },

    registerEventListeners() {
      const vm = this;
      const dropEventListener = document.addEventListener("CustomDragEnd", dropEvent => {
        vm.isBeingDraggedOver = false;
        vm.isGlobalDragInProgress = false;
        if (!dropEvent.detail || !dropEvent.detail.position) return;
        const {
          x,
          y
        } = dropEvent.detail.position;
        if (!vm.areCoordinatesInElement(x, y)) return;
        vm.$emit("onDrop", dropEvent.detail.payload);
      });
      vm.eventListeners.push(dropEventListener);
      const dragStartEventListener = document.addEventListener("CustomDragStart", () => {
        vm.isGlobalDragInProgress = true;
      });
      vm.eventListeners.push(dragStartEventListener);
      const touchMoveListener = document.addEventListener("touchmove", touchEvent => {
        if (!touchEvent || !touchEvent.target || !touchEvent.target.parentElement) return;
        if (touchEvent.target.parentElement.getAttribute("custom-draggable") !== "true") return;
        if (!touchEvent.targetTouches || touchEvent.targetTouches.length !== 1) return;
        const currentTouch = touchEvent.targetTouches[0];
        if (!currentTouch.pageX || !currentTouch.pageY) return;

        if (vm.areCoordinatesInElement(currentTouch.pageX, currentTouch.pageY)) {
          vm.isBeingDraggedOver = true;
        } else {
          vm.isBeingDraggedOver = false;
        }
      });
      vm.eventListeners.push(touchMoveListener);
      const dragEndListener = document.addEventListener("dragend", () => {
        vm.isBeingDraggedOver = false;
      });
      vm.eventListeners.push(dragEndListener);
      const mouseMoveEventListener = document.addEventListener("mousemove", mouseEvent => {
        if (!vm.isGlobalDragInProgress) return;

        if (!vm.areCoordinatesInElement(mouseEvent.pageX, mouseEvent.pageY)) {
          this.isBeingDraggedOver = false;
          return;
        }

        this.isBeingDraggedOver = true;
      });
      vm.eventListeners.push(mouseMoveEventListener);
    },

    deregisterEventListeners() {
      if (this.eventListeners.length) this.eventListeners.forEach(listener => document.removeEventListener(listener));
    }

  },

  mounted() {
    this.registerEventListeners();
  },

  unmounted() {
    this.deregisterEventListeners();
  }

};

/* script */
const __vue_script__ = script;
/* template */

var __vue_render__ = function () {
  var _obj;

  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', _vm._b({
    ref: "dropable",
    class: (_obj = {}, _obj[_vm.dragOverClass] = _vm.isBeingDraggedOver, _obj),
    attrs: {
      "id": "drop-handler"
    },
    on: {
      "drop": _vm.dropHandler,
      "dragenter": function ($event) {
        $event.preventDefault();
        return _vm.dragenterHandler.apply(null, arguments);
      },
      "dragover": function ($event) {
        $event.preventDefault();
      },
      "dragleave": _vm.dragleaveHandler
    }
  }, 'div', _vm.$attrs, false), [_vm._t("default")], 2);
};

var __vue_staticRenderFns__ = [];
/* style */

const __vue_inject_styles__ = undefined;
/* scoped */

const __vue_scope_id__ = undefined;
/* module identifier */

const __vue_module_identifier__ = undefined;
/* functional template */

const __vue_is_functional_template__ = false;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

const __vue_component__ = /*#__PURE__*/normalizeComponent({
  render: __vue_render__,
  staticRenderFns: __vue_staticRenderFns__
}, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__, __vue_module_identifier__, false, undefined, undefined, undefined);

var __vue_component__$1 = __vue_component__;

/* eslint-disable import/prefer-default-export */

var components = /*#__PURE__*/Object.freeze({
  __proto__: null,
  Draggable: __vue_component__$3,
  Dropable: __vue_component__$1
});

// Import vue components

const install = function installVueCrossPlatformDragDrop(Vue) {
  Object.entries(components).forEach(_ref => {
    let [componentName, component] = _ref;
    Vue.component(componentName, component);
  });
}; // Create module definition for Vue.use()

export { __vue_component__$3 as Draggable, __vue_component__$1 as Dropable, install as default };
